package com.jianshui.common.core.domain;

import java.util.HashMap;

import com.alibaba.fastjson.JSONObject;
import com.jianshui.common.constant.HttpStatus;
import com.jianshui.common.enums.ErrorCode;
import com.jianshui.common.utils.StringUtils;

/**
 * 操作消息提醒
 *
 * @author 简税平台
 */
public class AjaxResult extends HashMap<String, Object> {
    private static final long serialVersionUID = 1L;

    /**
     * 状态码
     */
    public static final String CODE_TAG = "code";

    /**
     * 返回内容
     */
    public static final String MSG_TAG = "msg";

    /**
     * 数据对象
     */
    public static final String DATA_TAG = "data";

    /**
     * 初始化一个新创建的 AjaxResult 对象，使其表示一个空消息。
     */
    public AjaxResult() {
    }

    /**
     * 初始化一个新创建的 AjaxResult 对象
     *
     * @param code 状态码
     * @param msg  返回内容
     */
    public AjaxResult(int code, String msg) {
        super.put(CODE_TAG, code);
        super.put(MSG_TAG, msg);
    }

    /**
     * 初始化一个新创建的 AjaxResult 对象
     *
     * @param code 状态码
     * @param msg  返回内容
     * @param data 数据对象
     */
    public AjaxResult(int code, String msg, Object data) {
        super.put(CODE_TAG, code);
        super.put(MSG_TAG, msg);
        if (StringUtils.isNotNull(data)) {
            super.put(DATA_TAG, data);
        }
    }

    /**
     * 返回成功消息
     *
     * @return 成功消息
     */
    public static AjaxResult success() {
        return AjaxResult.success("操作成功");
    }

    /**
     * 返回成功数据
     *
     * @return 成功消息
     */
    public static AjaxResult success(Object data) {
        return AjaxResult.success("操作成功", data);
    }

    /**
     * 返回成功消息
     *
     * @param msg 返回内容
     * @return 成功消息
     */
    public static AjaxResult success(String msg) {
        return AjaxResult.success(msg, null);
    }

    /**
     * 返回成功消息
     *
     * @param msg  返回内容
     * @param data 数据对象
     * @return 成功消息
     */
    public static AjaxResult success(String msg, Object data) {
        return new AjaxResult(HttpStatus.SUCCESS, msg, data);
    }

    /**
     * 返回错误消息
     *
     * @return
     */
    public static AjaxResult error() {
        return AjaxResult.error("操作失败");
    }

    /**
     * 返回错误消息
     *
     * @param msg 返回内容
     * @return 警告消息
     */
    public static AjaxResult error(String msg) {
        return AjaxResult.error(msg, null);
    }

    /**
     * 返回错误消息
     *
     * @param msg  返回内容
     * @param data 数据对象
     * @return 警告消息
     */
    public static AjaxResult error(String msg, Object data) {
        return new AjaxResult(HttpStatus.ERROR, msg, data);
    }


    public static AjaxResult error(Integer code, Object data) {
        return new AjaxResult(code, "error", data);
    }

    public static AjaxResult error(Object data) {
        return new AjaxResult(HttpStatus.ERROR, "error", data);
    }

    /**
     * 返回错误消息
     *
     * @return
     */
    public static AjaxResult error(ErrorCode code) {
        AjaxResult result = AjaxResult.error(code.getMsg());
        result.put(CODE_TAG, code.getCode());
        return result;
    }


    /**
     * 返回错误消息
     *
     * @param code 状态码
     * @param msg  返回内容
     * @return 警告消息
     */
    public static AjaxResult error(int code, String msg) {
        return new AjaxResult(code, msg, null);
    }

    /**
     * 方便链式调用
     *
     * @param key   键
     * @param value 值
     * @return 数据对象
     */
    @Override
    public AjaxResult put(String key, Object value) {
        super.put(key, value);
        return this;
    }

    /**
     * 判断一下是否成功
     *
     * @return
     */
    public boolean isSuccess() {
        Object code = this.get(CODE_TAG);
        Integer codeInt = null;
        if (code.getClass().equals(Integer.class)) {
            codeInt = (Integer) code;
        } else {
            codeInt = Integer.parseInt(code.toString());
        }
        if (codeInt == HttpStatus.SUCCESS) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 判断一下是否失败
     *
     * @return
     */
    public boolean isError() {
        return !this.isSuccess();
    }


    /**
     * 获取消息
     *
     * @return
     */
    public String getMsg() {
        return (String) this.get(MSG_TAG);
    }

    /**
     * 获取数据
     *
     * @return
     */
    public Object getData() {
        return this.get(DATA_TAG);
    }

    /**
     * 返回错误码
     *
     * @return
     */
    public int getCode() {
        return (int) this.get(CODE_TAG);
    }

    // data转json
    public JSONObject getJsonData() {
        if (this.get("data") == null) {
            return null;
        }
        // 判断一下类型是不是json，是json在转
        String dataClassName = this.get("data").getClass().getName();
        if (StringUtils.equals(dataClassName, "com.alibaba.fastjson.JSONObject")) {
            return (JSONObject) this.get("data");
        }

        if (StringUtils.equals(dataClassName, "java.lang.String")) {
            return JSONObject.parseObject((String) this.get("data"));
        }

        return null;
    }

    // data转json
    public <T> T getJsonData(Class<T> clazz) {
        return (T) this.get("data");
    }

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }
}
